@@ -21,30 +21,43 @@
 require 'node.rb'
 
 class DataTree
-	attr_reader :nodes, :tables, :enums
-	def initialize
-		@nodes = []
-		@enums = []
-		@tables = []
-	end#initialize
+  attr_reader :nodes, :tables, :enums, :types , :notifications
+  def initialize
+    @nodes = []
+    @enums = []
+    @tables = []
+    @notifications = []
+    @types = {}
+  end#initialize
 
-	def addEnum(enum)
-		@enums << enum
-	end#addEnum
+  def addType(type, basetype)
+    @types.store(type, basetype)
+  end#addEnum
 
-	def addNode(node)
-		@nodes << node
-	end#addNode
+  def addEnum(enum)
+    @enums << enum
+  end#addEnum
 
-	def addTable(table)
-		@tables << table
-	end#addNode
+  def addNode(node)
+    @nodes << node
+  end#addNode
+
+  def addTable(table)
+    @tables << table
+  end#addNode
+
+  def addNotification(notification)
+    @notifications << notification
+  end#addNode
 
   def to_s
-    @nodes.to_s + @enums.to_s + @tables.to_s
+    @nodes.to_s + @enums.to_s + @tables.to_s + @types.to_s + @notifications.to_s
   end#hash
 
-	def is_empty?
-		return (@nodes.size == 0 and @tables.size == 0)
-	end#empty?
+  def is_empty?
+    return (@nodes.size == 0 \
+            and @tables.size == 0 \
+            and @types.keys.size == 0 \
+            and @notifications.keys.size == 0)
+  end#empty?
 end#class DataTree
